@@ -25,6 +25,12 @@ from xdg import BaseDirectory
 
 from zeitgeist.client import ZeitgeistDBusInterface
 
+__all__ = [
+	"log",
+	"get_engine",
+	"constants"
+]
+
 logging.basicConfig(level=logging.DEBUG)
 log = logging.getLogger("zeitgeist.engine")
 
@@ -48,9 +54,10 @@ class _Constants:
 	DBUS_INTERFACE = ZeitgeistDBusInterface.INTERFACE_NAME
 	SIG_EVENT = "asaasay"
 	
-	# Extensions
-	DEFAULT_EXTENSIONS = [
-		"_zeitgeist.engine.extensions.blacklist.Blacklist"
-		]
+	# Required version of DB schema
+	CORE_SCHEMA="core"
+	CORE_SCHEMA_VERSION = 1
+	
+	USER_EXTENSION_PATH = os.path.join(DATA_PATH, "extensions")
 
 constants = _Constants()
