@@ -4,6 +4,7 @@
 #
 # Copyright © 2009-2010 Siegfried-Angel Gevatter Pujals <rainct@ubuntu.com>
 # Copyright © 2009 Mikkel Kamstrup Erlandsen <mikkel.kamstrup@gmail.com>
+# Copyright © 2010 Seif Lotfy <seif@lotfy.com>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -22,15 +23,13 @@ import dbus
 import dbus.service
 import logging
 
-from zeitgeist.datamodel import (Event, Subject, TimeRange, StorageState,
-	ResultType, NULL_EVENT)
+from zeitgeist.datamodel import TimeRange, StorageState, ResultType, NULL_EVENT
+from _zeitgeist.engine.datamodel import Event, Subject
 from _zeitgeist.engine import get_engine
 from _zeitgeist.engine.notify import MonitorManager
 from _zeitgeist.engine import constants
 from _zeitgeist.singleton import SingletonApplication
 
-_engine = get_engine()
-
 class RemoteInterface(SingletonApplication):
 	"""
 	Primary interface to the Zeitgeist engine. Used to update and query
@@ -43,7 +42,7 @@ class RemoteInterface(SingletonApplication):
 	:const:`org.gnome.zeitgeist.Engine`.
 	"""
 	_dbus_properties = {
-		"version": property(lambda self: (0, 3, 2, 0)),
+		"version": property(lambda self: (0, 5, 2)),
 	}
 	
 	# Initialization
@@ -51,6 +50,7 @@ class RemoteInterface(SingletonApplication):
 	def __init__(self, start_dbus=True, mainloop=None):
 		SingletonApplication.__init__(self)
 		self._mainloop = mainloop
+		self._engine = get_engine()
 		self._notifications = MonitorManager()
 	
 	# Private methods
@@ -64,8 +64,10 @@ class RemoteInterface(SingletonApplication):
 	# Reading stuff
 	
 	@dbus.service.method(constants.DBUS_INTERFACE,
-						in_signature="au", out_signature="a("+constants.SIG_EVENT+")")
-	def GetEvents(self, event_ids):
+						in_signature="au",
+						out_signature="a("+constants.SIG_EVENT+")",
+						sender_keyword="sender")
+	def GetEvents(self, event_ids, sender):
 		"""Get full event data for a set of event IDs
 		
 		Each event which is not found in the event log is represented
@@ -80,13 +82,14 @@ class RemoteInterface(SingletonApplication):
 		   :class:`Event` instances by calling *events = map(Event.new_for_struct, result)*
 		:rtype: A list of serialized events. DBus signature a(asaasay).
 		"""
-		return self._make_events_sendable(_engine.get_events(event_ids))
+		return self._make_events_sendable(self._engine.get_events(ids=event_ids,
+		    sender=sender))
 	
 	@dbus.service.method(constants.DBUS_INTERFACE,
-						in_signature="(xx)a("+constants.SIG_EVENT+")a("+constants.SIG_EVENT+")u",
+						in_signature="(xx)a("+constants.SIG_EVENT+")a("+constants.SIG_EVENT+")uuu",
 						out_signature="as")
 	def FindRelatedUris(self, time_range, event_templates,
-		result_event_templates, storage_state):
+		result_event_templates, storage_state, num_events, result_type):
 		"""Warning: This API is EXPERIMENTAL and is not fully supported yet.
 		
 		Get a list of URIs of subjects which frequently occur together
@@ -119,11 +122,16 @@ class RemoteInterface(SingletonApplication):
 		   available. The list of possible values is enumerated in the
 		   :class:`StorageState <zeitgeist.datamodel.StorageState>` class
 		:type storage_state: unsigned 32 bit integer, DBus signature :const:`u`
+		:param num_events: maximal amount of returned events
+		:type num_events: unsigned integer
+		:param result_type: unsigned integer 0 for relevancy 1 for recency
+		:type order: unsigned integer
 		:returns: A list of URIs matching the described criteria
 		:rtype: An array of strings, DBus signature :const:`as`.
 		"""
-		return _engine.find_related_uris(time_range, event_templates,
-			result_event_templates, storage_state)
+		event_templates = map(Event, event_templates)
+		return self._engine.find_related_uris(time_range, event_templates,
+			result_event_templates, storage_state, num_events, result_type)
 	
 	@dbus.service.method(constants.DBUS_INTERFACE,
 						in_signature="(xx)a("+constants.SIG_EVENT+")uuu",
@@ -141,6 +149,13 @@ class RemoteInterface(SingletonApplication):
 		subject then events will match the template if any one of their
 		subjects match any one of the subject templates.
 		
+		The fields uri, interpretation, manifestation, origin, and mimetype
+		can be prepended with an exclamation mark '!' in order to negate
+		the matching.
+		
+		The fields uri, origin, and mimetype can be prepended with an
+		asterisk '*' in order to do truncated matching.
+		
 		This method is intended for queries potentially returning a
 		large result set. It is especially useful in cases where only
 		a portion of the results are to be displayed at the same time
@@ -171,23 +186,22 @@ class RemoteInterface(SingletonApplication):
 		:param order: unsigned integer representing
 			a :class:`result type <zeitgeist.datamodel.ResultType>`
 		:type order: unsigned integer
-		:returns: Full event data for all the requested IDs, up to a maximum
-			of *num_events* events, sorted and grouped as defined by the
-			*result_type* parameter. The event data can be conveniently
-			converted into a list of :class:`Event` instances by calling
-			*events = map(Event.new_for_struct, result)*
-		:rtype: A list of serialized events. DBus signature a(asaasay).
+		:returns: An array containing the IDs of all matching events,
+			up to a maximum of *num_events* events. Sorted and grouped
+			as defined by the *result_type* parameter.
+		:rtype: Array of unsigned 32 bit integers
 		"""
 		time_range = TimeRange(time_range[0], time_range[1])
 		event_templates = map(Event, event_templates)
-		return _engine.find_eventids(time_range, event_templates, storage_state,
+		return self._engine.find_eventids(time_range, event_templates, storage_state,
 			num_events, result_type)
 
 	@dbus.service.method(constants.DBUS_INTERFACE,
 						in_signature="(xx)a("+constants.SIG_EVENT+")uuu",
-						out_signature="a("+constants.SIG_EVENT+")")
+						out_signature="a("+constants.SIG_EVENT+")",
+						sender_keyword="sender")
 	def FindEvents(self, time_range, event_templates, storage_state,
-			num_events, result_type):
+			num_events, result_type, sender):
 		"""Get events matching a given set of templates.
 		
 		The matching is done where unset fields in the templates
@@ -195,6 +209,13 @@ class RemoteInterface(SingletonApplication):
 		subject then events will match the template if any one of their
 		subjects match any one of the subject templates.
 		
+		The fields uri, interpretation, manifestation, origin, and mimetype
+		can be prepended with an exclamation mark '!' in order to negate
+		the matching.
+		
+		The fields uri, origin, and mimetype can be prepended with an
+		asterisk '*' in order to do truncated matching.
+		
 		In case you need to do a query yielding a large (or unpredictable)
 		result set and you only want to show some of the results at the
 		same time (eg., by paging them), use :meth:`FindEventIds`.
@@ -219,21 +240,25 @@ class RemoteInterface(SingletonApplication):
 		:param order: unsigned integer representing
 			a :class:`result type <zeitgeist.datamodel.ResultType>`
 		:type order: unsigned integer
-		:returns: An array containing the IDs of all matching events,
-			up to a maximum of *num_events* events. Sorted and grouped
-			as defined by the *result_type* parameter.
-		:rtype: Array of unsigned 32 bit integers
+		:returns: Full event data for all the requested IDs, up to a maximum
+			of *num_events* events, sorted and grouped as defined by the
+			*result_type* parameter. The event data can be conveniently
+			converted into a list of :class:`Event` instances by calling
+			*events = map(Event.new_for_struct, result)*
+		:rtype: A list of serialized events. DBus signature a(asaasay).
 		"""
 		time_range = TimeRange(time_range[0], time_range[1])
 		event_templates = map(Event, event_templates)
-		return self._make_events_sendable(_engine.find_events(time_range,
-			event_templates, storage_state, num_events, result_type))
+		return self._make_events_sendable(self._engine.find_events(time_range,
+			event_templates, storage_state, num_events, result_type, sender))
 
 	# Writing stuff
 	
 	@dbus.service.method(constants.DBUS_INTERFACE,
-						in_signature="a("+constants.SIG_EVENT+")", out_signature="au")
-	def InsertEvents(self, events):
+						in_signature="a("+constants.SIG_EVENT+")",
+						out_signature="au",
+						sender_keyword="sender")
+	def InsertEvents(self, events, sender):
 		"""Inserts events into the log. Returns an array containing the IDs
 		of the inserted events
 		
@@ -260,7 +285,7 @@ class RemoteInterface(SingletonApplication):
 		"""
 		if not events : return []
 		events = map(Event, events)
-		event_ids = _engine.insert_events(events)
+		event_ids = self._engine.insert_events(events, sender)
 		
 		_events = []
 		min_stamp = events[0].timestamp
@@ -280,21 +305,30 @@ class RemoteInterface(SingletonApplication):
 		
 		return event_ids
 	
-	@dbus.service.method(constants.DBUS_INTERFACE, in_signature="au", out_signature="")
-	def DeleteEvents(self, event_ids):
+	@dbus.service.method(constants.DBUS_INTERFACE,
+	                     in_signature="au",
+	                     out_signature="(xx)",
+	                     sender_keyword="sender")
+	def DeleteEvents(self, event_ids, sender):
 		"""Delete a set of events from the log given their IDs
 		
 		:param event_ids: list of event IDs obtained, for example, by calling
 			:meth:`FindEventIds`
 		:type event_ids: list of integers
 		"""
-		# FIXME: Notify monitors - how do we do this? //kamstrup
-		timestamps = _engine.delete_events(event_ids)
+		timestamps = self._engine.delete_events(event_ids, sender=sender)
 		if timestamps:
 			# We need to check the return value, as the events could already
-			# have been deleted before or the IDs might even have be invalid.
+			# have been deleted before or the IDs might even have been invalid.
 			self._notifications.notify_delete(
 			    TimeRange(timestamps[0], timestamps[1]), event_ids)
+		if timestamps is None:
+			# unknown event id, see doc of delete_events()
+			return (-1, -1)
+		timestamp_start, timestamp_end = timestamps
+		timestamp_start = timestamp_start if timestamp_start is not None else -1
+		timestamp_end = timestamp_end if timestamp_end is not None else -1
+		return (timestamp_start, timestamp_end)
 
 	@dbus.service.method(constants.DBUS_INTERFACE, in_signature="", out_signature="")
 	def DeleteLog(self):
@@ -304,13 +338,14 @@ class RemoteInterface(SingletonApplication):
 		content in one go. To delete specific subsets use
 		:meth:`FindEventIds` combined with :meth:`DeleteEvents`.
 		"""
-		_engine.delete_log()
+		self._engine.delete_log()
 	
 	@dbus.service.method(constants.DBUS_INTERFACE)
 	def Quit(self):
 		"""Terminate the running Zeitgeist engine process; use with caution,
 		this action must only be triggered with the user's explicit consent,
 		as it will affect all applications using Zeitgeist"""
+		self._engine.close()
 		if self._mainloop:
 			self._mainloop.quit()
 	
@@ -348,6 +383,9 @@ class RemoteInterface(SingletonApplication):
 		
 		The monitor object must implement the interface :ref:`org.gnome.zeitgeist.Monitor <org_gnome_zeitgeist_Monitor>`
 		
+		The monitor templates are matched exactly like described in
+		:meth:`FindEventIds`.
+		
 		:param monitor_path: DBus object path to the client side monitor object. DBus signature o.
 		:param time_range: A two-tuple with the time range monitored
 			events must fall within. Recall that time stamps are in
