@@ -7,7 +7,7 @@
 # Copyright © 2009 Natan Yellin <aantny@gmail.com>
 # Copyright © 2009 Seif Lotfy <seif@lotfy.com>
 # Copyright © 2009 Shane Fagan <shanepatrickfagan@yahoo.ie>
-# Copyright © 2009 Siegfried-Angel Gevatter Pujals <rainct@ubuntu.com>
+# Copyright © 2009-2010 Siegfried-Angel Gevatter Pujals <rainct@ubuntu.com>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -22,6 +22,7 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+from __future__ import with_statement
 import os
 import re
 import fnmatch
@@ -32,7 +33,7 @@ from xdg import BaseDirectory
 
 from zeitgeist import _config
 from zeitgeist.datamodel import Event, Subject, Interpretation, Manifestation, \
-	get_timestamp_for_now
+	DataSource, get_timestamp_for_now
 from _zeitgeist.loggers.zeitgeist_base import DataProvider
 
 log = logging.getLogger("zeitgeist.logger.datasources.recent")
@@ -86,7 +87,10 @@ DOCUMENT_MIMETYPES = [
 		u"application/x-abiword",
 		u"application/x-gnucash",
 		u"application/x-gnumeric",
-		SimpleMatch("application/x-java*"),
+		SimpleMatch(u"application/x-java*"),
+		SimpleMatch(u"*/x-tex"),
+		SimpleMatch(u"*/x-latex"),
+		SimpleMatch(u"*/x-dvi"),
 		u"text/plain"
 ]
 
@@ -110,49 +114,57 @@ VIDEO_MIMETYPES = [
 ]
 
 DEVELOPMENT_MIMETYPES = [
-		u"text/x-python",
-		u"application/x-perl",
-		u"application/x-sql",
-		u"text/x-java",
-		u"text/x-csrc",
-		u"text/x-c++src",
-		u"text/css",
-		U"application/x-shellscript",
-		U"application/x-object",
-		u"application/x-php",
-		u"application/x-java-archive",
-		u"text/html",
-		u"application/xml",
-		u"text/x-dsrc",
-		u"text/x-pascal",
-		u"text/x-patch",
-		u"application/x-csh",
-		u"text/x-eiffel",
-		u"application/x-fluid",
-		u"text/x-chdr",
-		u"text/x-idl",
+		u"application/ecmascript",
 		u"application/javascript",
-		u"text/x-lua",
-		u"text/x-objcsrc",
-		u"application/x-m4",
-		u"text/x-ocaml",
-		u"text/x-tcl",
-		u"text/x-vhdl",
+		u"application/x-csh",
+		u"application/x-designer",
+		u"application/x-desktop",
+		u"application/x-dia-diagram",
+		u"application/x-fluid",
+		u"application/x-glade",
 		u"application/xhtml+xml",
+		u"application/x-java-archive",
+		u"application/x-m4",
+		u"application/xml",
+		u"application/x-object",
+		u"application/x-perl",
+		u"application/x-php",
+		u"application/x-ruby",
+		u"application/x-shellscript",
+		u"application/x-sql",
+		u"text/css",
+		u"text/html",
+		u"text/x-c",
+		u"text/x-c++",
+		u"text/x-chdr",
+		u"text/x-copying",
+		u"text/x-credits",
+		u"text/x-csharp",
+		u"text/x-c++src",
+		u"text/x-csrc",
+		u"text/x-dsrc",
+		u"text/x-eiffel",
 		u"text/x-gettext-translation",
 		u"text/x-gettext-translation-template",
-		u"application/x-glade",
-		u"application/x-designer",
-		u"text/x-makefile",
-		u"text/x-sql",
-		u"application/x-desktop",
-		u"text/x-csharp",
-		u"application/ecmascript",
 		u"text/x-haskell",
-		u"text/x-copying"
+		u"text/x-idl",
+		u"text/x-java",
+		u"text/x-lisp",
+		u"text/x-lua",
+		u"text/x-makefile",
+		u"text/x-objcsrc",
+		u"text/x-ocaml",
+		u"text/x-pascal",
+		u"text/x-patch",
+		u"text/x-python",
+		u"text/x-sql",
+		u"text/x-tcl",
+		u"text/x-troff",
+		u"text/x-vala",
+		u"text/x-vhdl",
 ]
 
-ALL_MIMETYPES = DOCUMENT_MIMETYPES + IMAGE_MIMETYPES + AUDIO_MIMETYPES +\
+ALL_MIMETYPES = DOCUMENT_MIMETYPES + IMAGE_MIMETYPES + AUDIO_MIMETYPES + \
 				VIDEO_MIMETYPES + DEVELOPMENT_MIMETYPES
 
 class MimeTypeSet(set):
@@ -192,40 +204,98 @@ class RecentlyUsedManagerGtk(DataProvider):
 	FILTERS = {
 		# dict of name as key and the matching mimetypes as value
 		# if the value is None this filter matches all mimetypes
-		u"Document": MimeTypeSet(*DOCUMENT_MIMETYPES),
-		u"Image": MimeTypeSet(*IMAGE_MIMETYPES),
-		u"Music": MimeTypeSet(*AUDIO_MIMETYPES),
-		u"Video": MimeTypeSet(*VIDEO_MIMETYPES),
-		u"SourceCode": MimeTypeSet(*DEVELOPMENT_MIMETYPES),
+		"DOCUMENT": MimeTypeSet(*DOCUMENT_MIMETYPES),
+		"IMAGE": MimeTypeSet(*IMAGE_MIMETYPES),
+		"AUDIO": MimeTypeSet(*AUDIO_MIMETYPES),
+		"VIDEO": MimeTypeSet(*VIDEO_MIMETYPES),
+		"SOURCE_CODE": MimeTypeSet(*DEVELOPMENT_MIMETYPES),
 	}
 	
-	def __init__(self):
-		DataProvider.__init__(self, name="Recently Used Documents")
+	def __init__(self, client):
+		DataProvider.__init__(self,
+			unique_id="com.zeitgeist-project,datahub,recent",
+			name="Recently Used Documents",
+			description="Logs events from GtkRecentlyUsed",
+			event_templates=[Event.new_for_values(interpretation=i) for i in (
+				Interpretation.CREATE_EVENT,
+				Interpretation.ACCESS_EVENT,
+				Interpretation.MODIFY_EVENT
+			)],
+			client=client)
+		self._load_data_sources_registry()
 		self.recent_manager = recent_manager()
 		self.recent_manager.set_limit(-1)
 		self.recent_manager.connect("changed", lambda m: self.emit("reload"))
 		self.config.connect("configured", lambda m: self.emit("reload"))
-		self._timestamp_last_run = 0
+	
+	def _load_data_sources_registry(self):
+		self._ignore_apps = {}
+		def _data_source_registered(datasource):
+			for tmpl in datasource[DataSource.EventTemplates]:
+				actor = tmpl[0][Event.Actor]
+				if actor:
+					if not actor in self._ignore_apps:
+						self._ignore_apps[actor] = set()
+					interp = tmpl[0][Event.Interpretation]
+					if interp:
+						self._ignore_apps[actor].add(interp)
+		for datasource in self._registry.GetDataSources():
+			_data_source_registered(datasource)
+		self._registry.connect("DataSourceRegistered", _data_source_registered)
 	
 	@staticmethod
-	def _find_desktop_file_for_application(application):
+	def _desktop_file_matches_app(filename, application, mimetype):
+		""" Checks whether the given .desktop file represents the indicated
+		application.
+		
+		If a mimetype is also given, only .desktop files listing it as
+		supported will be considered. This is needed to differentiate
+		between the different OpenOffice.org components, for instance.
+		"""
+		
+		try:
+			with open(filename) as desktopfile:
+				_exec = False
+				_mime = False
+				for line in desktopfile:
+					if line.startswith("Exec=") and \
+					line.split("=", 1)[-1].strip().split()[0] == application:
+						if mimetype is None or _mime:
+							return True
+						else:
+							_exec = True
+					elif line.startswith("MimeType=") and mimetype in \
+					line.split("=", 1)[-1].split(";"):
+						if _exec:
+							return True
+						else:
+							_mime = True
+						
+		except IOError:
+			pass # file may be a broken symlink (LP: #523761)
+		except Exception, e:
+			log.warning('Corrupt .desktop file: %s', filename)
+		
+		return False
+	
+	@staticmethod
+	def _find_desktop_file_for_application(application, mimetype):
 		""" Searches for a .desktop file for the given application in
 		$XDG_DATADIRS and returns the path to the found file. If no file
 		is found, returns None.
 		"""
-		
 		desktopfiles = \
 			list(BaseDirectory.load_data_paths("applications", "%s.desktop" % application))
 		if desktopfiles:
 			return unicode(desktopfiles[0])
 		else:
+			is_match = RecentlyUsedManagerGtk._desktop_file_matches_app # local stuff is accessed faster
 			for path in BaseDirectory.load_data_paths("applications"):
 				for filename in (name for name in os.listdir(path) if name.endswith(".desktop")):
 					fullname = os.path.join(path, filename)
-					for line in open(fullname):
-						if line.startswith("Exec") and \
-						line.split("=", 1)[-1].strip().split()[0] == application:
-							return unicode(fullname)
+					if (is_match(fullname, application, mimetype)):
+						return unicode(fullname)
+		
 		return None
 	
 	def _get_interpretation_for_mimetype(self, mimetype):
@@ -235,51 +305,67 @@ class RecentlyUsedManagerGtk(DataProvider):
 				matching_filter = filter_name
 				break
 		if matching_filter:
-			return getattr(Interpretation, matching_filter.upper()).uri
-		return Interpretation.UNKNOWN.uri
+			return getattr(Interpretation, matching_filter).uri
+		return ""
 	
 	def _get_items(self):
-		timestamp_last_run = get_timestamp_for_now()
+		# We save the start timestamp to avoid race conditions
+		last_seen = get_timestamp_for_now()
 		
 		events = []
 		
 		for (num, info) in enumerate(self.recent_manager.get_items()):
-			if info.exists() and not info.get_private_hint() and not info.get_uri_display().startswith("/tmp/"):
-				subject = Subject.new_for_values(
-					uri = unicode(info.get_uri()),
-					interpretation = self._get_interpretation_for_mimetype(
-						unicode(info.get_mime_type())),
-					manifestation = Manifestation.FILE.uri,
-					text = info.get_display_name(),
-					mimetype = unicode(info.get_mime_type()),
-					origin = info.get_uri().rpartition("/")[0]
-				)
-				
+			uri = info.get_uri()
+			if info.exists() and not info.get_private_hint() and not uri.startswith("file:///tmp/"):
 				last_application = info.last_application().strip()
 				application = info.get_application_info(last_application)[0].split()[0]
-				desktopfile = self._find_desktop_file_for_application(application)
-				times = (
-					(info.get_added() * 1000, Interpretation.CREATE_EVENT.uri),
-					(info.get_visited() * 1000, Interpretation.VISIT_EVENT.uri),
-					(info.get_modified() * 1000, Interpretation.MODIFY_EVENT.uri)
+				mimetype = unicode(info.get_mime_type())
+				if application in ("ooffice", "soffice"):
+					# Special case OpenOffice.org *sigh*
+					desktopfile = self._find_desktop_file_for_application("ooffice", mimetype)
+				else:
+					desktopfile = self._find_desktop_file_for_application(application, None)
+				if not desktopfile:
+					continue
+				actor = u"application://%s" % os.path.basename(desktopfile)
+				
+				subject = Subject.new_for_values(
+					uri = unicode(uri),
+					interpretation = self._get_interpretation_for_mimetype(
+						unicode(info.get_mime_type())),
+					manifestation = Manifestation.FILE_DATA_OBJECT.uri,
+					text = info.get_display_name(),
+					mimetype = mimetype,
+					origin = uri.rpartition("/")[0]
 				)
 				
+				times = set()
+				for meth, interp in (
+					(info.get_added, Interpretation.CREATE_EVENT.uri),
+					(info.get_visited, Interpretation.ACCESS_EVENT.uri),
+					(info.get_modified, Interpretation.MODIFY_EVENT.uri)
+					):
+					if actor not in self._ignore_apps or \
+						(self._ignore_apps[actor] and
+						interp not in self._ignore_apps[actor]):
+						times.add((meth() * 1000, interp))
+				
 				is_new = False
 				for timestamp, use in times:
-					if timestamp <= self._timestamp_last_run:
+					if timestamp <= self._last_seen:
 						continue
 					is_new = True
 					events.append(Event.new_for_values(
 						timestamp = timestamp,
 						interpretation = use,
 						manifestation = Manifestation.USER_ACTIVITY.uri,
-						actor = desktopfile or u"",
+						actor = actor,
 						subjects = [subject]
 						))
 			if num % 50 == 0:
 				self._process_gobject_events()
-		self._timestamp_last_run = timestamp_last_run
+		self._last_seen = last_seen
 		return events
 
 if enabled:
-	__datasource__ = RecentlyUsedManagerGtk()
+	__datasource__ = RecentlyUsedManagerGtk
