@@ -3,7 +3,7 @@
 # Zeitgeist
 #
 # Copyright © 2009 Seif Lotfy <seif@lotfy.com>
-# Copyright © 2009 Siegfried-Angel Gevatter Pujals <rainct@ubuntu.com>
+# Copyright © 2009-2010 Siegfried-Angel Gevatter Pujals <rainct@ubuntu.com>
 # Copyright © 2009 Natan Yellin <aantny@gmail.com>
 # Copyright © 2009 Alex Graveley <alex@beatniksoftware.com>
 # Copyright © 2009 Markus Korn <thekorn@gmx.de>
@@ -25,6 +25,7 @@ from threading import Thread
 import gobject
 import logging
 
+from zeitgeist.datamodel import DataSource
 from _zeitgeist.loggers.zeitgeist_setup_service import _Configuration, DefaultConfiguration
 
 class DataProvider(gobject.GObject, Thread):
@@ -33,15 +34,30 @@ class DataProvider(gobject.GObject, Thread):
 		"reload" : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()),
 	}
 	
-	def __init__(self, name, config=None):
+	def __init__(self, unique_id, name, description="", event_templates=[],
+		client=None, config=None):
 		
 		# Initialize superclasses
 		Thread.__init__(self)
 		gobject.GObject.__init__(self)
 		
 		self._name = name
+		self._client = client
 		self._ctx = gobject.main_context_default()
 		
+		if client:
+			self._registry = self._client.get_extension("DataSourceRegistry",
+				"data_source_registry")
+			try:
+				self._last_seen = [ds[DataSource.LastSeen] for ds in \
+					self._registry.GetDataSources() if \
+					ds[DataSource.UniqueId] == unique_id][0] - 1800000
+				# We substract 30 minutes to make sure no events get missed.
+			except IndexError:
+				self._last_seen = 0
+			self._enabled = self._registry.RegisterDataSource(unique_id, name,
+				description, event_templates)
+		
 		if not config:
 			self.config = DefaultConfiguration(self._name)
 		else:
@@ -53,6 +69,11 @@ class DataProvider(gobject.GObject, Thread):
 		return self._name
 	
 	def get_items(self):
+		if not self._enabled:
+			return []
+		# FIXME: We need to figure out what to do with this configuration stuff
+		# Maybe merge it into the DataSource registry so that everyone
+		# can benefit from it, or just throw it out.
 		if not self.config.isConfigured() or not self.config.enabled:
 			logging.warning("'%s' isn't enabled or configured." % \
 				self.config.get_internal_name())
