@@ -2,7 +2,7 @@
 
 # Zeitgeist
 #
-# Copyright © 2009 Siegfried-Angel Gevatter Pujals <rainct@ubuntu.com>
+# Copyright © 2009-2010 Siegfried-Angel Gevatter Pujals <rainct@ubuntu.com>
 # Copyright © 2009 Mikkel Kamstrup Erlandsen <mikkel.kamstrup@gmail.com>
 # Copyright © 2009 Markus Korn <thekorn@gmx.de>
 #
@@ -26,6 +26,7 @@ import logging
 import os.path
 import sys
 import logging
+import inspect
 
 from xml.dom.minidom import parseString as minidom_parse
 
@@ -39,23 +40,12 @@ SIG_EVENT = "asaasay"
 logging.basicConfig(level=logging.DEBUG)
 log = logging.getLogger("zeitgeist.client")
 
-class ZeitgeistDBusInterface(dbus.Interface):
-	""" Central DBus interface to the zeitgeist engine
-	
-	There does not necessarily have to be one single instance of this
-	interface class, but all instances should share the same state
-	(like use the same bus and be connected to the same proxy). This is
-	achieved by extending the `Borg Pattern` as described by Alex Martelli	
-	"""
-	__shared_state = {}
-	
-	BUS_NAME = "org.gnome.zeitgeist.Engine"
-	INTERFACE_NAME = "org.gnome.zeitgeist.Log"
-	OBJECT_PATH = "/org/gnome/zeitgeist/log/activity"
-	
+class _DBusInterface(object):
+	"""Wrapper around dbus.Interface adding convenience methods."""
+
 	@staticmethod
 	def get_members(introspection_xml):
-		"""Parses the xml context returned by Introspect() and returns
+		"""Parses the XML context returned by Introspect() and returns
 		a tuple, where the first item is a list of all methods and the
 		second one a list of all signals for the related interface
 		"""
@@ -69,96 +59,132 @@ class ZeitgeistDBusInterface(dbus.Interface):
 		except ValueError:
 			pass
 		return methods, signals
-	
-	@classmethod
-	def get_session_bus(cls):
-		"""Returns the bus used by the interface.
-		
-		If there is no bus set, the '_bus' attribute is set to
-		dbus.SessionBus() and returned
+
+	def _disconnection_safe(self, meth):
 		"""
-		return cls.__shared_state.setdefault("_bus", dbus.SessionBus())
-	
-	@classmethod
-	def _get_proxy(cls):
-		"""Returns the proxy instance used by the interface.
-		
-		If the current interface has no proxy object set, it tries to
-		generate one. If this fails because no zeitgeist-daemon is
-		running a RuntimeError will be raised
+		Executes the given method. If it fails because the D-Bus connection
+		was lost, attempts to recover it and executes the method again.
 		"""
 		try:
-			return cls.__shared_state["proxy_object"]
-		except KeyError, e:
-			bus = cls.get_session_bus()
-			try:
-				cls.__shared_state["proxy_object"] = bus.get_object(
-					cls.BUS_NAME,
-					cls.OBJECT_PATH
-				)
-			except dbus.exceptions.DBusException, e:
-				if e.get_dbus_name() == "org.freedesktop.DBus.Error.ServiceUnknown":
-					raise RuntimeError(("Found no running instance of the "
-						"Zeitgeist daemon: %s") % e.get_dbus_message())
-				else:
-					raise
+			return meth()
+		except dbus.exceptions.DBusException, e:
+			error = e.get_dbus_name()
+			if error == "org.freedesktop.DBus.Error.ServiceUnknown":
+				self.__proxy = dbus.SessionBus().get_object(
+					self.__iface.requested_bus_name, self.__object_path)
+				self.__iface = dbus.Interface(self.__proxy,
+					self.__interface_name)
+				return meth()
 			else:
-				introspection_xml = cls.__shared_state["proxy_object"].Introspect()
-				methods, signals = cls.get_members(introspection_xml)
-				cls.__shared_state["__methods"] = methods
-				cls.__shared_state["__signals"] = signals
-			return cls.__shared_state["proxy_object"]
-	
-	@classmethod
-	def connect(cls, signal, callback, arg0=None):
-		"""Connect a callback to a signal of the current proxy instance """
-		proxy = cls._get_proxy()
-		if signal not in cls.__shared_state["__signals"]:
-			raise TypeError("unknown signal name: %s" %signal)
-		if arg0 is None:
-			proxy.connect_to_signal(
-				signal,
-				callback,
-				dbus_interface=cls.INTERFACE_NAME
-			)
-		else:
-			# TODO: This is ugly and limited to 1 argument. Find a better
-			# way to do it.
-			proxy.connect_to_signal(
-				signal,
-				callback,
-				dbus_interface=cls.INTERFACE_NAME,
-				arg0=arg0
-			)
-			
-	@classmethod
-	def connect_exit(cls, callback):
-		"""executes callback when the RemoteInterface exists"""
-		bus = cls.get_session_bus()
-		bus_obj = bus.get_object(dbus.BUS_DAEMON_IFACE, dbus.BUS_DAEMON_PATH)
+				raise
+
+	def __getattr__(self, name):
+		if name not in self.__methods:
+			raise TypeError("Unknown method name: %s" % name)
+		def _ProxyMethod(*args, **kwargs):
+			"""
+			Method wrapping around a D-Bus call, which attempts to recover
+			the connection to Zeitgeist if it got lost.
+			"""
+			return self._disconnection_safe(lambda:
+				getattr(self.__iface, name)(*args, **kwargs))
+		return _ProxyMethod
+
+	def connect(self, signal, callback, **kwargs):
+		"""Connect a callback to a signal of the current proxy instance."""
+		if signal not in self.__signals:
+			raise TypeError("Unknown signal name: %s" % signal)
+		self.__proxy.connect_to_signal(
+			signal,
+			callback,
+			dbus_interface=self.__interface_name,
+			**kwargs)
+
+	def connect_exit(self, callback):
+		"""Executes callback when the RemoteInterface exists"""
+		bus_obj = dbus.SessionBus().get_object(dbus.BUS_DAEMON_IFACE,
+			dbus.BUS_DAEMON_PATH)
 		bus_obj.connect_to_signal(
 			"NameOwnerChanged",
 			lambda *args: callback(),
 			dbus_interface=dbus.BUS_DAEMON_IFACE,
-			arg0=cls.BUS_NAME, #only match dying zeitgeist engines
-			arg2="", #only match services with no new owner
+			arg0=self.__iface.requested_bus_name, # only match what we want
+			arg2="", # only match services with no new owner
 		)
+
+	@property
+	def proxy(self):
+		return self.__proxy
+
+	def __init__(self, proxy, interface_name, object_path):
+		self.__proxy = proxy
+		self.__interface_name = interface_name
+		self.__object_path = object_path
+		self.__iface = dbus.Interface(proxy, interface_name)
+		self.__methods, self.__signals = self.get_members(proxy.Introspect())
+
+class ZeitgeistDBusInterface(object):
+	""" Central DBus interface to the Zeitgeist engine
+	
+	There does not necessarily have to be one single instance of this
+	interface class, but all instances should share the same state
+	(like use the same bus and be connected to the same proxy). This is
+	achieved by extending the `Borg Pattern` as described by Alex Martelli	
+	"""
+	__shared_state = {}
+	
+	BUS_NAME = "org.gnome.zeitgeist.Engine"
+	INTERFACE_NAME = "org.gnome.zeitgeist.Log"
+	OBJECT_PATH = "/org/gnome/zeitgeist/log/activity"
+	
+	def __getattr__(self, name):
+		return getattr(self.__shared_state["dbus_interface"], name)
+	
+	def version(self):
+		"""Returns the API version"""
+		dbus_interface = self.__shared_state["dbus_interface"]
+		return dbus_interface._disconnection_safe(lambda:
+			dbus_interface.proxy.get_dbus_method("Get",
+				dbus_interface=dbus.PROPERTIES_IFACE)(self.INTERFACE_NAME,
+					"version"))
+	
+	def get_extension(cls, name, path, busname=None):
+		""" Returns an interface to the given extension.
 		
-	@classmethod
-	def version(cls):
-		""" get the API version """
-		proxy = cls._get_proxy()
-		return proxy.get_dbus_method("Get",
-			dbus_interface=dbus.PROPERTIES_IFACE)(cls.INTERFACE_NAME, "version")
+		Example usage:
+			>> reg = get_extension("DataSourceRegistry", "data_source_registry")
+			>> reg.RegisterDataSource(...)
+		"""
+		if busname:
+			busname = "org.gnome.zeitgeist.%s" % busname
+		else:
+			busname = cls.BUS_NAME
+		if not name in cls.__shared_state["extension_interfaces"]:
+			interface_name = "org.gnome.zeitgeist.%s" % name
+			object_path = "/org/gnome/zeitgeist/%s" % path
+			proxy = dbus.SessionBus().get_object(busname, object_path)
+			iface = _DBusInterface(proxy, interface_name, object_path)
+			iface.BUS_NAME = busname
+			iface.INTERFACE_NAME = interface_name
+			iface.OBJECT_PATH = object_path
+			cls.__shared_state["extension_interfaces"][name] = iface
+		return cls.__shared_state["extension_interfaces"][name]
 	
 	def __init__(self):
-		self.__dict__ = self.__shared_state
-		proxy = self._get_proxy()
-		dbus.Interface.__init__(
-				self,
-				proxy,
-				self.INTERFACE_NAME
-		)
+		if not "dbus_interface" in self.__shared_state:
+			try:
+				proxy = dbus.SessionBus().get_object(self.BUS_NAME,
+					self.OBJECT_PATH)
+			except dbus.exceptions.DBusException, e:
+				if e.get_dbus_name() == "org.freedesktop.DBus.Error.ServiceUnknown":
+					raise RuntimeError(
+						"Found no running zeitgeist-daemon instance: %s" % \
+						e.get_dbus_message())
+				else:
+					raise
+			self.__shared_state["extension_interfaces"] = {}
+			self.__shared_state["dbus_interface"] = _DBusInterface(proxy,
+				self.INTERFACE_NAME, self.OBJECT_PATH)
 
 class Monitor(dbus.service.Object):
 	"""
@@ -266,8 +292,25 @@ class ZeitgeistClient:
 	always the right thing to do. If you really want to do synchronous
 	DBus calls use the raw DBus API found in the ZeitgeistDBusInterface class.
 	"""
+	
+	@staticmethod
+	def get_event_and_extra_arguments(arguments):
+		""" some methods of :class:`ZeitgeistClient` take a variable
+		number of arguments, where one part of the arguments are used
+		to build one :class:`Event` instance and the other part
+		is forwarded to another method. This function returns an event
+		and the remaining arguments."""
+		kwargs = {}
+		for arg in _FIND_EVENTS_FOR_TEMPLATES_ARGS:
+			if arg in arguments:
+				kwargs[arg] = arguments.pop(arg)
+		ev = Event.new_for_values(**arguments)
+		return ev, kwargs
+	
 	def __init__ (self):
 		self._iface = ZeitgeistDBusInterface()
+		self._registry = self._iface.get_extension("DataSourceRegistry",
+			"data_source_registry")
 	
 	def _safe_error_handler(self, error_handler, *args):
 		if error_handler is not None:
@@ -470,11 +513,10 @@ class ZeitgeistClient:
 		allowed keywords are the same as the ones allowed by
 		:meth:`Event.new_for_values() <zeitgeist.datamodel.Event.new_for_values>`.
 		"""
-		ev = Event.new_for_values(**kwargs)
-		
+		ev, arguments = self.get_event_and_extra_arguments(kwargs)
 		self.find_event_ids_for_templates([ev],
 						ids_reply_handler,
-						**kwargs)
+						**arguments)
 	
 	def find_events_for_templates (self,
 					event_templates,
@@ -555,9 +597,10 @@ class ZeitgeistClient:
 		Alias for :meth:`find_events_for_templates`, for use when only
 		one template is needed.
 		"""
-		self.find_event_ids_for_templates([event_template],
+		ev, arguments = self.get_event_and_extra_arguments(kwargs)
+		self.find_events_for_templates([ev],
 						events_reply_handler,
-						**kwargs)
+						**arguments)
 	
 	def find_events_for_values(self, events_reply_handler, **kwargs):
 		"""
@@ -568,11 +611,10 @@ class ZeitgeistClient:
 		allowed keywords are the same as the ones allowed by
 		:meth:`Event.new_for_values() <zeitgeist.datamodel.Event.new_for_values>`.
 		"""
-		ev = Event.new_for_values(**kwargs)
-		
+		ev, arguments = self.get_event_and_extra_arguments(kwargs)
 		self.find_events_for_templates([ev],
 						events_reply_handler,
-						**kwargs)
+						**arguments)
 	
 	def get_events (self, event_ids, events_reply_handler, error_handler=None):
 		"""
@@ -630,15 +672,17 @@ class ZeitgeistClient:
 		runnning.
 		"""
 		self._check_list_or_tuple(event_ids)
-		self._check_members(event_ids, int)
+		# we need dbus.UInt32 here as long as dbus.UInt32 is not a subtype
+		# of int, this might change in the future, see docstring of dbus.UInt32
+		self._check_members(event_ids, (int, dbus.UInt32))
 		
 		self._iface.DeleteEvents(event_ids,
 					reply_handler=self._safe_reply_handler(reply_handler),
 					error_handler=self._safe_error_handler(error_handler))
 	
 	def find_related_uris_for_events(self, event_templates, uris_reply_handler,
-		time_range = None, result_event_templates=[],
-		storage_state=StorageState.Any, error_handler=None):
+		error_handler=None, time_range = None, result_event_templates=[],
+		storage_state=StorageState.Any, num_events=10, result_type=0):
 		"""
 		Warning: This API is EXPERIMENTAL and is not fully supported yet.
 		
@@ -657,6 +701,10 @@ class ZeitgeistClient:
 		    as subjects of events matching these templates
 		:param storage_state: The returned URIs must have this
 		    :class:`storage state <zeitgeist.datamodel.StorageState>`
+		:param num_events: number of related uris you want to have returned
+		:param result_type: sorting of the results by 
+			0 for relevancy
+			1 for recency
 		:param error_handler: An optional callback in case of errors.
 		    Must take a single argument being the error raised by the
 		    server. The default behaviour in case of errors is to call
@@ -668,18 +716,19 @@ class ZeitgeistClient:
 				"Reply handler not callable, found %s" % uris_reply_handler)
 		
 		if time_range is None:
-			time_range = TimeRange.always()
-		
+			time_range = TimeRange.until_now()
+			
 		self._iface.FindRelatedUris(time_range, event_templates,
-			result_event_templates, storage_state,
+			result_event_templates, storage_state, num_events, result_type,
 			reply_handler=self._safe_reply_handler(uris_reply_handler),
 			error_handler=self._safe_error_handler(error_handler,
 			                                       uris_reply_handler,
-			                                       []))
+			                                       [])
+			)
 	
 	def find_related_uris_for_uris(self, subject_uris, uris_reply_handler,
 		time_range=None, result_event_templates=[],
-		storage_state=StorageState.Any, error_handler=None):
+		storage_state=StorageState.Any,  num_events=10, result_type=0, error_handler=None):
 		"""
 		Warning: This API is EXPERIMENTAL and is not fully supported yet.
 		
@@ -695,6 +744,8 @@ class ZeitgeistClient:
 		                                  time_range=time_range,
 		                                  result_event_templates=result_event_templates,
 		                                  storage_state=storage_state,
+		                                  num_events = num_events,
+		                                  result_type = result_type,
 		                                  error_handler=error_handler)
 	
 	def install_monitor (self, time_range, event_templates,
@@ -794,7 +845,31 @@ class ZeitgeistClient:
 		self._iface.RemoveMonitor(path,
 		                          reply_handler=reply_handler,
 		                          error_handler=error_handler)
+	
+	def register_data_source(self, unique_id, name, description, event_templates):
+		"""
+		Register a data-source as currently running. If the data-source was
+		already in the database, its metadata (name, description and
+		event_templates) are updated.
 		
+		If the data-source registry isn't enabled, do nothing.
+		
+		The optional event_templates is purely informational and serves to
+		let data-source management applications and other data-sources know
+		what sort of information you log.
+		
+		:param unique_id: unique ASCII string identifying the data-source
+		:param name: data-source name (may be translated)
+		:param description: data-source description (may be translated)
+		:param event_templates: list of
+			:class:`Event <zeitgeist.datamodel.Event>` templates.
+		"""
+		# TODO: Make it possible to access the return value!
+		self._registry.RegisterDataSource(unique_id, name, description,
+			event_templates,
+			reply_handler=self._void_reply_handler,
+			error_handler=self._void_reply_handler) # Errors are ignored
+	
 	def _check_list_or_tuple(self, collection):
 		"""
 		Raise a ValueError unless 'collection' is a list or tuple
@@ -829,3 +904,6 @@ class ZeitgeistClient:
 		
 		if callable(normal_reply_handler):
 			normal_reply_handler(normal_reply_data)
+
+_FIND_EVENTS_FOR_TEMPLATES_ARGS = inspect.getargspec(
+	ZeitgeistClient.find_events_for_templates)[0]
